import pytest

from titan import resources as res
from titan.blueprint import _merge_pointers
from titan.resources.resource import ResourcePointer
from titan.enums import ResourceType

# from titan.identifiers import parse_URN
# from titan.privs import AccountPriv, DatabasePriv, GrantedPrivilege
# from titan.resource_name import ResourceName


def test_merge_account_scoped_pointers():
    warehouse = res.Warehouse(name="test_warehouse")
    warehouse_pointer = ResourcePointer(name="test_warehouse", resource_type=ResourceType.WAREHOUSE)
    resources = [warehouse, warehouse_pointer]
    merged = _merge_pointers(resources)
    assert len(merged) == 1
    assert merged[0] is warehouse


def test_merge_implied_database_into_actual():
    database = res.Database(name="test_database")
    schema = res.Schema(name="test_database.someschema")
    resources = [database, schema.container]
    merged = _merge_pointers(resources)
    assert len(merged) == 1
    assert merged[0] is database


def test_merge_moves_children():
    schema = res.Schema(name="someschema")
    task = res.Task(name="someschema.sometask")
    assert isinstance(task.container, ResourcePointer)
    resources = [schema, task.container]
    merged = _merge_pointers(resources)
    assert len(merged) == 1
    assert merged[0] is schema
    assert task in schema.items()
    assert task.container is schema


def test_merge_deep_tree():
    database = res.Database(name="test_database")
    schema = res.Schema(name="test_database.someschema")
    task = res.Task(name="test_database.someschema.sometask")
    account_scoped_resources = [database, schema.container]
    merged = _merge_pointers(account_scoped_resources)
    assert len(merged) == 1

    db_scoped_resources = [schema, task.container]
    merged = _merge_pointers(db_scoped_resources)
    assert len(merged) == 1
    assert merged[0] is schema
    assert task.container is schema
    assert task.container.container is database


def test_merge_throw_away_duplicates():
    database = res.Database(name="test_database")
    resources = [database, database]
    merged = _merge_pointers(resources)
    assert len(merged) == 1
    assert merged[0] is database



import logging
from dataclasses import dataclass, field

from ..enums import ParseableEnum, ResourceType
from ..props import BoolProp, EnumProp, IntProp, Props, StringListProp, StringProp, TagsProp
from ..resource_name import ResourceName
from ..scope import AccountScope
from .resource import NamedResource, Resource, ResourceSpec
from .role import Role
from .tag import TaggableResource

logger = logging.getLogger("titan")


class UserType(ParseableEnum):
    PERSON = "PERSON"
    SERVICE = "SERVICE"
    LEGACY_SERVICE = "LEGACY_SERVICE"
    NULL = "NULL"


@dataclass(unsafe_hash=True)
class _User(ResourceSpec):
    name: ResourceName
    owner: Role = "USERADMIN"
    password: str = field(default=None, metadata={"fetchable": False})
    login_name: ResourceName = None
    display_name: str = None
    first_name: str = None
    middle_name: str = None
    last_name: str = None
    email: str = None
    must_change_password: bool = None
    disabled: bool = False
    days_to_expiry: int = field(default=None, metadata={"known_after_apply": True})
    mins_to_unlock: int = None
    default_warehouse: str = None
    default_namespace: str = None
    default_role: str = None
    default_secondary_roles: list[str] = None
    mins_to_bypass_mfa: int = None
    rsa_public_key: str = None
    rsa_public_key_2: str = None
    comment: str = None
    network_policy: str = None
    type: UserType = "NULL"

    def __post_init__(self):
        super().__post_init__()

        if self.type is None:
            self.type = UserType.NULL

        if self.type == UserType.SERVICE:
            if self.first_name is not None:
                raise ValueError("First name is not supported for service users")
            if self.middle_name is not None:
                raise ValueError("Middle name is not supported for service users")
            if self.last_name is not None:
                raise ValueError("Last name is not supported for service users")
            if self.password is not None:
                raise ValueError("Password is not supported for service users")
            if self.must_change_password is not None:
                raise ValueError("Must change password is not supported for service users")
            if self.mins_to_bypass_mfa is not None:
                raise ValueError("Mins to bypass MFA is not supported for service users")

        else:
            if self.login_name is None or self.login_name == "":
                self.login_name = ResourceName(str(self.name).upper())
            if self.display_name is None:
                self.display_name = self.name._name
            if self.must_change_password is None:
                self.must_change_password = False


class User(NamedResource, TaggableResource, Resource):
    """
    Description:
        A user in Snowflake.

    Snowflake Docs:
        https://docs.snowflake.com/en/sql-reference/sql/create-user

    Fields:
        name (string, required): The name of the user.
        owner (string or Role): The owner of the user. Defaults to "USERADMIN".
        password (string): The password of the user.
        login_name (string): The login name of the user. Defaults to the name in uppercase.
        display_name (string): The display name of the user. Defaults to the name in lowercase.
        first_name (string): The first name of the user.
        middle_name (string): The middle name of the user.
        last_name (string): The last name of the user.
        email (string): The email of the user.
        must_change_password (bool): Whether the user must change their password. Defaults to False.
        disabled (bool): Whether the user is disabled. Defaults to False.
        days_to_expiry (int): The number of days until the user's password expires.
        mins_to_unlock (int): The number of minutes until the user's account is unlocked.
        default_warehouse (string): The default warehouse for the user.
        default_namespace (string): The default namespace for the user.
        default_role (string): The default role for the user.
        default_secondary_roles (list): The default secondary roles for the user.
        mins_to_bypass_mfa (int): The number of minutes until the user can bypass Multi-Factor Authentication.
        rsa_public_key (string): The RSA public key for the user.
        rsa_public_key_2 (string): The RSA public key for the user.
        comment (string): A comment for the user.
        network_policy (string): The network policy for the user.
        type (string or UserType): The type of the user. Defaults to "NULL".
        tags (dict): Tags for the user.

    Python:

        ```python
        user = User(
            name="some_user",
            owner="USERADMIN",
            email="some.user@example.com",
            type="PERSON",
        )
        ```

    Yaml:

        ```yaml
        users:
          - name: some_user
            owner: USERADMIN
            email: some.user@example.com
            type: PERSON
        ```

    """

    resource_type = ResourceType.USER
    props = Props(
        password=StringProp("password"),
        login_name=StringProp("login_name"),
        display_name=StringProp("display_name"),
        first_name=StringProp("first_name"),
        middle_name=StringProp("middle_name"),
        last_name=StringProp("last_name"),
        email=StringProp("email"),
        must_change_password=BoolProp("must_change_password"),
        disabled=BoolProp("disabled"),
        days_to_expiry=IntProp("days_to_expiry"),
        mins_to_unlock=IntProp("mins_to_unlock"),
        default_warehouse=StringProp("default_warehouse"),
        default_namespace=StringProp("default_namespace"),
        default_role=StringProp("default_role"),
        default_secondary_roles=StringListProp("default_secondary_roles", parens=True),
        mins_to_bypass_mfa=IntProp("mins_to_bypass_mfa"),
        rsa_public_key=StringProp("rsa_public_key"),
        rsa_public_key_2=StringProp("rsa_public_key_2"),
        comment=StringProp("comment"),
        network_policy=StringProp("network_policy"),
        type=EnumProp("type", UserType),
        tags=TagsProp(),
    )
    scope = AccountScope()
    spec = _User

    def __init__(
        self,
        name: str,
        owner: str = "USERADMIN",
        password: str = None,
        login_name: str = None,
        display_name: str = None,
        first_name: str = None,
        middle_name: str = None,
        last_name: str = None,
        email: str = None,
        must_change_password: bool = None,
        disabled: bool = False,
        days_to_expiry: int = None,
        mins_to_unlock: int = None,
        default_warehouse: str = None,
        default_namespace: str = None,
        default_role: str = None,
        default_secondary_roles: list[str] = None,
        mins_to_bypass_mfa: int = None,
        rsa_public_key: str = None,
        rsa_public_key_2: str = None,
        comment: str = None,
        network_policy: str = None,
        type: str = None,
        tags: dict[str, str] = None,
        **kwargs,
    ):

        user_type = kwargs.pop("user_type", None)
        if user_type:
            logging.warning("The 'user_type' parameter is deprecated. Use 'type' instead.")
            if type is None:
                type = user_type
            else:
                raise ValueError("Both 'type' and 'user_type' parameters are set. Use only 'type'.")

        super().__init__(name, **kwargs)
        self._data: _User = _User(
            name=self._name,
            owner=owner,
            password=password,
            login_name=login_name,
            display_name=display_name,
            first_name=first_name,
            middle_name=middle_name,
            last_name=last_name,
            email=email,
            must_change_password=must_change_password,
            disabled=disabled,
            days_to_expiry=days_to_expiry,
            mins_to_unlock=mins_to_unlock,
            default_warehouse=default_warehouse,
            default_namespace=default_namespace,
            default_role=default_role,
            default_secondary_roles=default_secondary_roles,
            mins_to_bypass_mfa=mins_to_bypass_mfa,
            rsa_public_key=rsa_public_key,
            rsa_public_key_2=rsa_public_key_2,
            comment=comment,
            network_policy=network_policy,
            type=type,
        )
        self.set_tags(tags)

    @property
    def owner(self):
        return self._data.owner



from dataclasses import dataclass

from ..enums import ParseableEnum, ResourceType
from ..props import BoolProp, EnumProp, Props, StringListProp, StringProp
from ..resource_name import ResourceName
from ..scope import AccountScope
from .resource import NamedResource, Resource, ResourceSpec
from .role import Role


class StorageProvider(ParseableEnum):
    S3 = "S3"
    AZURE = "AZURE"
    GCS = "GCS"


@dataclass(unsafe_hash=True)
class _S3StorageIntegration(ResourceSpec):
    name: ResourceName
    enabled: bool
    storage_aws_role_arn: str
    storage_allowed_locations: list[str]
    storage_blocked_locations: list[str] = None
    storage_provider: StorageProvider = StorageProvider.S3
    storage_aws_object_acl: str = None
    type: str = "EXTERNAL_STAGE"
    owner: Role = "ACCOUNTADMIN"
    comment: str = None

    def __post_init__(self):
        super().__post_init__()
        if self.type != "EXTERNAL_STAGE":
            raise ValueError("Type must be 'EXTERNAL_STAGE' for S3StorageIntegration")


class S3StorageIntegration(NamedResource, Resource):
    """
    Description:
        Manages the integration of Snowflake with S3 storage.

    Snowflake Docs:
        https://docs.snowflake.com/en/sql-reference/sql/create-storage-integration

    Fields:
        name (string, required): The name of the storage integration.
        enabled (bool, required): Whether the storage integration is enabled. Defaults to True.
        storage_aws_role_arn (string, required): The AWS IAM role ARN to access the S3 bucket.
        storage_allowed_locations (list, required): A list of allowed locations for storage in the format 's3://<bucket>/<path>/'.
        storage_blocked_locations (list): A list of blocked locations for storage in the format 's3://<bucket>/<path>/'. Defaults to an empty list.
        storage_aws_object_acl (string): The ACL policy for objects stored in S3. Defaults to 'bucket-owner-full-control'.
        type (string): The type of storage integration. Defaults to 'EXTERNAL_STAGE'.
        owner (string or Role): The owner role of the storage integration. Defaults to 'ACCOUNTADMIN'.
        comment (string): An optional comment about the storage integration.

    Python:

        ```python
        s3_storage_integration = S3StorageIntegration(
            name="some_s3_storage_integration",
            enabled=True,
            storage_aws_role_arn="arn:aws:iam::123456789012:role/MyS3AccessRole",
            storage_allowed_locations=["s3://mybucket/myfolder/"],
            storage_blocked_locations=["s3://mybucket/myblockedfolder/"],
            storage_aws_object_acl="bucket-owner-full-control",
            comment="This is a sample S3 storage integration."
        )
        ```

    Yaml:

        ```yaml
        s3_storage_integrations:
          - name: some_s3_storage_integration
            enabled: true
            storage_aws_role_arn: "arn:aws:iam::123456789012:role/MyS3AccessRole"
            storage_allowed_locations:
              - "s3://mybucket/myfolder/"
            storage_blocked_locations:
              - "s3://mybucket/myblockedfolder/"
            storage_aws_object_acl: "bucket-owner-full-control"
            comment: "This is a sample S3 storage integration."
        ```
    """

    resource_type = ResourceType.STORAGE_INTEGRATION
    props = Props(
        type=StringProp("type"),
        storage_provider=EnumProp("storage_provider", [StorageProvider.S3]),
        storage_aws_role_arn=StringProp("storage_aws_role_arn"),
        storage_aws_object_acl=StringProp("storage_aws_object_acl"),
        enabled=BoolProp("enabled"),
        storage_allowed_locations=StringListProp("storage_allowed_locations", parens=True),
        storage_blocked_locations=StringListProp("storage_blocked_locations", parens=True),
        comment=StringProp("comment"),
    )
    scope = AccountScope()
    spec = _S3StorageIntegration

    def __init__(
        self,
        name: str,
        enabled: bool,
        storage_aws_role_arn: str,
        storage_allowed_locations: list[str],
        storage_blocked_locations: list[str] = None,
        storage_aws_object_acl: str = None,
        owner: str = "ACCOUNTADMIN",
        comment: str = None,
        **kwargs,
    ):
        kwargs.pop("storage_provider", None)
        kwargs.pop("type", None)
        super().__init__(name, **kwargs)
        self._data: _S3StorageIntegration = _S3StorageIntegration(
            name=self._name,
            enabled=enabled,
            storage_aws_role_arn=storage_aws_role_arn,
            storage_allowed_locations=storage_allowed_locations,
            storage_blocked_locations=storage_blocked_locations,
            storage_aws_object_acl=storage_aws_object_acl,
            owner=owner,
            comment=comment,
        )


@dataclass(unsafe_hash=True)
class _GCSStorageIntegration(ResourceSpec):
    name: ResourceName
    enabled: bool
    storage_allowed_locations: list[str]
    storage_blocked_locations: list[str] = None
    storage_provider: StorageProvider = StorageProvider.GCS
    type: str = "EXTERNAL_STAGE"
    owner: Role = "ACCOUNTADMIN"
    comment: str = None

    def __post_init__(self):
        super().__post_init__()
        if self.type != "EXTERNAL_STAGE":
            raise ValueError("Type must be 'EXTERNAL_STAGE' for GCSStorageIntegration")


class GCSStorageIntegration(NamedResource, Resource):
    """
    Description:
        Manages the integration of Google Cloud Storage (GCS) as an external stage for storing data.

    Snowflake Docs:
        https://docs.snowflake.com/en/sql-reference/sql/create-storage-integration

    Fields:
        name (string, required): The name of the storage integration.
        enabled (bool, required): Specifies whether the storage integration is enabled.
        storage_allowed_locations (list): A list of allowed GCS locations for data storage.
        storage_blocked_locations (list): A list of blocked GCS locations for data storage.
        owner (string or Role): The owner role of the storage integration. Defaults to 'ACCOUNTADMIN'.
        comment (string): An optional comment about the storage integration.

    Python:

        ```python
        gcs_storage_integration = GCSStorageIntegration(
            name="some_gcs_storage_integration",
            enabled=True,
            storage_allowed_locations=['gcs://bucket/path/'],
            storage_blocked_locations=['gcs://bucket/blocked_path/']
        )
        ```

    Yaml:

        ```yaml
        gcs_storage_integrations:
          - name: some_gcs_storage_integration
            enabled: true
            storage_allowed_locations:
              - 'gcs://bucket/path/'
            storage_blocked_locations:
              - 'gcs://bucket/blocked_path/'
        ```
    """

    resource_type = ResourceType.STORAGE_INTEGRATION
    props = Props(
        type=StringProp("type"),
        storage_provider=EnumProp("storage_provider", [StorageProvider.GCS]),
        enabled=BoolProp("enabled"),
        storage_allowed_locations=StringListProp("storage_allowed_locations", parens=True),
        storage_blocked_locations=StringListProp("storage_blocked_locations", parens=True),
        comment=StringProp("comment"),
    )
    scope = AccountScope()
    spec = _GCSStorageIntegration

    def __init__(
        self,
        name: str,
        enabled: bool,
        storage_allowed_locations: list[str],
        storage_blocked_locations: list[str] = None,
        owner: str = "ACCOUNTADMIN",
        comment: str = None,
        **kwargs,
    ):
        kwargs.pop("storage_provider", None)
        kwargs.pop("type", None)
        super().__init__(name, **kwargs)
        self._data: _GCSStorageIntegration = _GCSStorageIntegration(
            name=self._name,
            enabled=enabled,
            storage_allowed_locations=storage_allowed_locations,
            storage_blocked_locations=storage_blocked_locations,
            owner=owner,
            comment=comment,
        )


@dataclass(unsafe_hash=True)
class _AzureStorageIntegration(ResourceSpec):
    name: ResourceName
    enabled: bool
    azure_tenant_id: str
    storage_allowed_locations: list[str]
    storage_blocked_locations: list[str] = None
    storage_provider: StorageProvider = StorageProvider.AZURE
    type: str = "EXTERNAL_STAGE"
    owner: Role = "ACCOUNTADMIN"
    comment: str = None

    def __post_init__(self):
        super().__post_init__()
        if self.type != "EXTERNAL_STAGE":
            raise ValueError("Type must be 'EXTERNAL_STAGE' for _AzureStorageIntegration")


class AzureStorageIntegration(NamedResource, Resource):
    """
    Description:
        Represents an Azure storage integration in Snowflake, which allows Snowflake to access external cloud storage using Azure credentials.

    Snowflake Docs:
        https://docs.snowflake.com/en/sql-reference/sql/create-storage-integration

    Fields:
        name (string, required): The name of the storage integration.
        enabled (bool, required): Specifies whether the storage integration is enabled.
        azure_tenant_id (string, required): The Azure tenant ID associated with the storage integration.
        storage_allowed_locations (list): The cloud storage locations that are allowed.
        storage_blocked_locations (list): The cloud storage locations that are blocked.
        owner (string or Role): The owner role of the storage integration. Defaults to "ACCOUNTADMIN".
        comment (string): A comment about the storage integration.

    Python:

        ```python
        azure_storage_integration = AzureStorageIntegration(
            name="some_azure_storage_integration",
            enabled=True,
            azure_tenant_id="some_tenant_id",
            storage_allowed_locations=["azure://somebucket/somepath/"],
            storage_blocked_locations=["azure://someotherbucket/somepath/"],
            comment="This is an Azure storage integration."
        )
        ```

    Yaml:

        ```yaml
        azure_storage_integrations:
          - name: some_azure_storage_integration
            enabled: true
            azure_tenant_id: some_tenant_id
            storage_allowed_locations:
              - azure://somebucket/somepath/
            storage_blocked_locations:
              - azure://someotherbucket/somepath/
            comment: This is an Azure storage integration.
        ```
    """

    resource_type = ResourceType.STORAGE_INTEGRATION
    props = Props(
        type=StringProp("type"),
        storage_provider=EnumProp("storage_provider", [StorageProvider.AZURE]),
        azure_tenant_id=StringProp("azure_tenant_id"),
        enabled=BoolProp("enabled"),
        storage_allowed_locations=StringListProp("storage_allowed_locations", parens=True),
        storage_blocked_locations=StringListProp("storage_blocked_locations", parens=True),
        comment=StringProp("comment"),
    )
    scope = AccountScope()
    spec = _AzureStorageIntegration

    def __init__(
        self,
        name: str,
        enabled: bool,
        azure_tenant_id: str,
        storage_allowed_locations: list[str],
        storage_blocked_locations: list[str] = None,
        owner: str = "ACCOUNTADMIN",
        comment: str = None,
        **kwargs,
    ):
        kwargs.pop("storage_provider", None)
        kwargs.pop("type", None)
        super().__init__(name, **kwargs)
        self._data: _AzureStorageIntegration = _AzureStorageIntegration(
            name=self._name,
            enabled=enabled,
            azure_tenant_id=azure_tenant_id,
            storage_allowed_locations=storage_allowed_locations,
            storage_blocked_locations=storage_blocked_locations,
            owner=owner,
            comment=comment,
        )


StorageIntegrationMap = {
    StorageProvider.S3: S3StorageIntegration,
    StorageProvider.GCS: GCSStorageIntegration,
    StorageProvider.AZURE: AzureStorageIntegration,
}


def _resolver(data: dict):
    return StorageIntegrationMap[StorageProvider(data["storage_provider"])]


Resource.__resolvers__[ResourceType.STORAGE_INTEGRATION] = _resolver



from dataclasses import dataclass, field

from ..enums import ParseableEnum, ResourceType
from ..props import (
    BoolProp,
    EnumProp,
    IntProp,
    Props,
    SchemaProp,
    StringProp,
    TagsProp,
)
from ..resource_name import ResourceName
from ..scope import SchemaScope
from .column import Column
from .external_volume import ExternalVolume
from .resource import NamedResource, Resource, ResourceSpec
from .role import Role
from .tag import TaggableResource


class StorageSerializationPolicy(ParseableEnum):
    COMPATIBLE = "COMPATIBLE"
    OPTIMIZED = "OPTIMIZED"


@dataclass(unsafe_hash=True)
class _SnowflakeIcebergTable(ResourceSpec):
    name: ResourceName
    columns: list[Column]
    owner: Role = "SYSADMIN"
    external_volume: ExternalVolume = None
    catalog: str = "SNOWFLAKE"
    base_location: str = None
    catalog_sync: str = None
    storage_serialization_policy: StorageSerializationPolicy = StorageSerializationPolicy.OPTIMIZED
    data_retention_time_in_days: int = 1
    max_data_extension_time_in_days: int = 14
    # Snowflake does not currently provide a way to check if change tracking is enabled for iceberg tables
    change_tracking: bool = field(default=None, metadata={"fetchable": False})
    default_ddl_collation: str = None
    comment: str = None

    def __post_init__(self):
        super().__post_init__()
        if self.columns is None:
            raise ValueError("columns can't be None")
        if len(self.columns) == 0:
            raise ValueError("columns can't be empty")


class SnowflakeIcebergTable(NamedResource, TaggableResource, Resource):

    resource_type = ResourceType.ICEBERG_TABLE
    props = Props(
        columns=SchemaProp(),
        external_volume=StringProp("external_volume"),
        catalog=StringProp("catalog"),
        base_location=StringProp("base_location"),
        catalog_sync=StringProp("catalog_sync"),
        storage_serialization_policy=EnumProp("storage_serialization_policy", StorageSerializationPolicy),
        data_retention_time_in_days=IntProp("data_retention_time_in_days"),
        max_data_extension_time_in_days=IntProp("max_data_extension_time_in_days"),
        change_tracking=BoolProp("change_tracking"),
        default_ddl_collation=StringProp("default_ddl_collation"),
        comment=StringProp("comment"),
        tags=TagsProp(),
    )
    scope = SchemaScope()
    spec = _SnowflakeIcebergTable

    def __init__(
        self,
        name: str,
        columns: list[Column],
        owner: str = "SYSADMIN",
        external_volume: str = None,
        catalog: str = "SNOWFLAKE",
        base_location: str = None,
        catalog_sync: str = None,
        storage_serialization_policy: str = "OPTIMIZED",
        data_retention_time_in_days: int = 1,
        max_data_extension_time_in_days: int = 14,
        change_tracking: bool = None,
        default_ddl_collation: str = None,
        comment: str = None,
        tags: dict[str, str] = None,
        **kwargs,
    ):

        if "lifecycle" not in kwargs:
            lifecycle = {
                "ignore_changes": "columns",
            }
            kwargs["lifecycle"] = lifecycle

        super().__init__(name, **kwargs)

        self._data: _SnowflakeIcebergTable = _SnowflakeIcebergTable(
            name=self._name,
            columns=columns,
            owner=owner,
            external_volume=external_volume,
            catalog=catalog,
            base_location=base_location,
            catalog_sync=catalog_sync,
            storage_serialization_policy=storage_serialization_policy,
            data_retention_time_in_days=data_retention_time_in_days,
            max_data_extension_time_in_days=max_data_extension_time_in_days,
            change_tracking=change_tracking,
            default_ddl_collation=default_ddl_collation,
            comment=comment,
        )
        if self._data.external_volume:
            self._requires(self._data.external_volume)
        self.set_tags(tags)



import pytest

from tests.helpers import get_json_fixtures
from titan import resources as res
from titan.enums import ResourceType
from titan.resources.resource import convert_role_ref
from titan.resources.resource import ResourcePointer

JSON_FIXTURES = list(get_json_fixtures())


@pytest.fixture(
    params=JSON_FIXTURES,
    ids=[resource_cls.__name__ for resource_cls, _ in JSON_FIXTURES],
    scope="function",
)
def json_fixture(request):
    resource_cls, data = request.param
    yield resource_cls, data


def test_resource_requires(json_fixture):
    role = res.Role(name="dummy")
    resource_cls, data = json_fixture
    resource = resource_cls(**data)
    assert role.requires(resource) is None
    assert resource in role.refs


def test_convert_role_ref():
    role_ref = convert_role_ref("dummy")
    assert isinstance(role_ref, ResourcePointer)
    assert role_ref.resource_type == ResourceType.ROLE
    assert role_ref.name == "dummy"

    database_role_ref = convert_role_ref("dummy.database_role")
    assert isinstance(database_role_ref, ResourcePointer)
    assert database_role_ref.resource_type == ResourceType.DATABASE_ROLE
    assert database_role_ref.name == "database_role"
    assert database_role_ref.container.name == "dummy"

    role = res.Role(name="dummy")
    assert convert_role_ref(role) == role
    database_role = res.DatabaseRole(name="dummy", database="dummy_db")
    assert convert_role_ref(database_role) == database_role
    role_ptr = ResourcePointer(name="dummy", resource_type=ResourceType.ROLE)
    assert convert_role_ref(role_ptr) == role_ptr
    database_role_ptr = ResourcePointer(name="dummy", resource_type=ResourceType.DATABASE_ROLE)
    assert convert_role_ref(database_role_ptr) == database_role_ptr

    with pytest.raises(TypeError):
        convert_role_ref(None)

    with pytest.raises(TypeError):
        convert_role_ref(111)

    with pytest.raises(TypeError):
        convert_role_ref(ResourcePointer(name="dummy", resource_type=ResourceType.DATABASE))
